import time
import hmac
import hashlib


def generate_nonce():
    return int(round(time.time() * 1000))


def generate_signature(secret, verb, path, nonce, data):
    if isinstance(data, (bytes, bytearray)):
        data = data.decode('utf8')
    message = verb + path + str(nonce) + data
    signature = hmac.new(
        bytes(secret, 'utf8'),
        bytes(message, 'utf8'),
        digestmod=hashlib.sha256)
    return signature.hexdigest()



from urllib.parse import urlparse
import logging
import requests
from . import auth_utils


class _BitmexNonceAuth(requests.auth.AuthBase):
    def __init__(self, api_key, api_secret):
        self.api_key = api_key
        self.api_secret = api_secret

    def __call__(self, r):
        nonce = auth_utils.generate_nonce()

        parsed_url = urlparse(r.url)
        path = parsed_url.path
        if parsed_url.query:
            path += '?' + parsed_url.query

        r.headers.update({
            'api-nonce': str(nonce),
            'api-signature': auth_utils.generate_signature(self.api_secret, r.method, path, nonce,
                                                           r.body or ''),
            'api-key': self.api_key,
        })

        return r


class RestError(Exception):
    def __init__(self, code, text, message=None):
        super().__init__(f'HTTP {code}: {message or ""}')
        self.code = code
        self.text = text
        self.message = message


class RestBadRequestError(RestError):
    pass


class RestRateLimitExceededError(RestError):
    def __init__(self, code, text, retry_after):
        super().__init__(code, text)
        self.retry_after = retry_after


class RestTemporaryDownError(RestError):
    pass


class RestSession:
    def __init__(self, domain, api_key, api_secret):
        self._base_url = f'https://{domain}/api/v1'
        self._session = requests.Session()
        self._session.headers.update({
            'user-agent': 'angou-bitmex-yo',
            'content-type': 'application/json',
            'accept': 'application/json',
        })
        self._auth = _BitmexNonceAuth(api_key, api_secret)
        self.logger = logging.getLogger('angou_bitmex')

    def request(self, verb, path, query=None, postdict=None):
        self.logger.debug('%s %s query=%s postdict=%s', verb, path, query, postdict)

        url = self._base_url + path

        req = requests.Request(verb, url, json=postdict, auth=self._auth, params=query)
        prepared_req = self._session.prepare_request(req)
        resp = self._session.send(prepared_req)

        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as ex:
            if resp is None:
                raise ex
            elif resp.status_code == 400:
                error = resp.json()['error']
                message = error['message'] if error else ''
                raise RestBadRequestError(resp.status_code, resp.text, message)
            elif resp.status_code == 429:
                retry_after = None
                try:
                    retry_after = int(resp.headers['Retry-After'])
                except (ValueError, KeyError):
                    pass
                raise RestRateLimitExceededError(resp.status_code, resp.text, retry_after)
            elif resp.status_code == 503:
                raise RestTemporaryDownError(resp.status_code, resp.text)
            else:
                raise RestError(resp.status_code, resp.text)
        else:
            return resp.json()



from setuptools import setup


VERSION = '1'


setup(name='angou_bitmex',
      version=VERSION,
      description='Lightweight and suckless BitMEX REST API client library',
      url='http://github.com/angou-exchange-utils/angou-bitmex',
      download_url='https://github.com/angou-exchange-utils/angou-bitmex/tarball/v' + VERSION,
      author='shdown',
      author_email='shdownnine@gmail.com',
      license='LGPLv3',
      packages=['angou_bitmex'],
      install_requires=['requests'],
      python_requires='>=3',
      platforms=['any'],
      keywords='angou bitmex crypto exchange',
      classifiers=[
          'Development Status :: 1 - Planning',
          'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
          'Programming Language :: Python :: 3 :: Only',
          'Topic :: Internet',
          'Topic :: Software Development :: Libraries',
      ],
      zip_safe=False)
